#include "Arduino.h"
#include "theSecrets.h"
#include "WiFi.h"
#include "lwip_hooks.h"

#include "Firewall.hpp"
#include "API.hpp"

fw::Firewall *firewall;
fw::API *firewallApi;

int lwip_hook_ip4_input(struct pbuf *pbuf, struct netif *input_netif)
{
    // Firewall is not setup yet
    if (firewall != NULL)
    {
        if (firewall->is_packet_allowed(pbuf))
            return 0;
        else
        {
            pbuf_free(pbuf);
            return 1;
        }
    }
    return 0;
}

void initFirewall(const String ip)
{
    firewall = new fw::Firewall();
    firewallApi = new fw::API(firewall, username, password, ip);
}

void handle_wifi_events(WiFiEvent_t event, WiFiEventInfo_t info)
{
    String ip;
    Serial.print("[WiFi] ");
    switch (event)
    {
    case ARDUINO_EVENT_WIFI_STA_START:
        Serial.println("connecting...");
        break;
    case ARDUINO_EVENT_WIFI_STA_CONNECTED:
        Serial.println("connected");
        break;
    case ARDUINO_EVENT_WIFI_STA_DISCONNECTED:
        Serial.println("disconnected");
        break;
    case ARDUINO_EVENT_WIFI_STA_GOT_IP:
        ip = WiFi.localIP().toString();
        Serial.print("IP: ");
        Serial.println(ip);
        initFirewall(ip);
        break;
    default:
        Serial.print("other event: ");
        Serial.println(event);
    }
}

void setup()
{
    Serial.begin(115200);
    WiFi.mode(WIFI_STA);
    WiFi.onEvent(handle_wifi_events, ARDUINO_EVENT_MAX);
    WiFi.begin(ssid, psk);

    // fix for https://github.com/espressif/arduino-esp32/issues/4732
    WiFi.config(((u32_t)0x0UL), ((u32_t)0x0UL), ((u32_t)0x0UL));
}

void loop()
{
    // https://docs.espressif.com/projects/arduino-esp32/en/latest/esp-idf_component.html
    sleep(1);
}